<?php
namespace Teknisa\FIS\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager\SQL\ManagerImpl as DataSourceManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\FIS\Services\Fis55000ParametrizacaoExportacaoMunicipal as Fis55000Service;
use Teknisa\FIS\Utilities\MessagesFiscal;
use Teknisa\Libs\Util\NewCode;


class Fis55000ParametrizacaoExportacaoMunicipal 
{
    const IDRELACIONAMENTO = 'relacionamento';
    
    protected $dataSource = 'parammuniserv_produto';
    protected $dataSourceToSave = 'parammuniserv';
    
    // protected $service;
    protected $crudServie;
    protected $sessionManager;
    protected $dataSourceManager;
    protected $environment;
    protected $newCode;
    
    const PRODUTOS_DIFRENTES = "produtos";
    const PRODUTOS_IGUAIS = "existentes";
    const PRODUTOS_BASE = "produtos_base";
    
    public function __construct(/*Fis55000Service $service,*/ CrudService $crudServie, $sessionManager, DataSourceManager $dataSourceManager, $environment, NewCode $newCode)
    {
        // $this->service = $service;
        $this->crudServie = $crudServie;
        $this->sessionManager = $sessionManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->newCode = $newCode;
    }
    
    /*public function cadastroAutomatico(Request\Row $request, Response $response)
    {  
        try {
            $reqRow = $request->getRow();
            $idRelacionamento = $request->getParameter(self::IDRELACIONAMENTO);
            
            $inscricoes = $reqRow['INSCRICOES'][0] == 'T' ? 
                $this->getAllInscricoes($reqRow['INSCRICOES'][1])
                : $reqRow['INSCRICOES'];
            
            switch($idRelacionamento) {
                case 'SOBREESCREVER':
                    $this->fazRelacionamentoIguais($inscricoes);
                    $this->fazRelacionamentoDiferentes($inscricoes);
                    break;
                case 'NAOSOBREESCREVER':
                    $this->fazRelacionamentoIguais($inscricoes);
                    break;
                default:
                    $this->limpaCache();
                    
                    $produtos = $reqRow['PRODUTOS'][0] == 'T' ? 
                            $this->getAllProdutos($reqRow['PRODUTOS'][1])      
                            : $reqRow['PRODUTOS'];
                            
                    $this->setInCache(self::PRODUTOS_BASE, $produtos);
                    
                    if ($this->naoSobreescreve($produtos, $inscricoes)) {
                        $this->fazRelacionamentoIguais($inscricoes);
                    } else {
                        $response->addMessage(new Message(MessagesFiscal::PRODUTOS_JA_PARAMETRIZADOS, 'W'));
                    }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }*/
    
    public function cadastroAutomatico(Request\Row $request, Response $response)
    {  
        try {
            $reqRow = $request->getRow();
            $idRelacionamento = $request->getParameter(self::IDRELACIONAMENTO);
            $produtos = $reqRow['PRODUTOS'][0] == 'T' ? 
                $this->getAllProdutos($reqRow['PRODUTOS'][1])      
                : $reqRow['PRODUTOS'];
            $this->setInCache(self::PRODUTOS_BASE, $produtos);
            
            $inscricoes = $reqRow['INSCRICOES'][0] == 'T' ? 
                $this->getAllInscricoes($reqRow['INSCRICOES'][1])
                : $reqRow['INSCRICOES'];
            
            switch($idRelacionamento) {
                case 'SOBREESCREVER':
                    $this->fazRelacionamentoIguais($inscricoes);
                    $this->fazRelacionamentoDiferentes($inscricoes);
                    break;
                case 'NAOSOBREESCREVER':
                    $this->fazRelacionamentoIguais($inscricoes);
                    break;
                default:
                    $this->limpaCache();
                    
                    
                    
                    $this->fazRelacionamentoIguais($inscricoes);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function naoSobreescreve(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            
            $inscricoes = $row['INSCRICOES'][0] == 'T' ? $this->getAllInscricoes($row['INSCRICOES'][1]) : $row['INSCRICOES'];
            $produtos = $row['PRODUTOS'][0] == 'T' ? $this->getAllProdutos($row['PRODUTOS'][1]) : $row['PRODUTOS'];
            
            $produtosMapiados = array_map(function($produto) {
                return $produto['CDPRODSERV'];
            }, $produtos);
            $existentes = $this->getProdusoParametrizadosInscricoes($produtosMapiados, $inscricoes);
            if (count($existentes) > 0) {
                $response->addMessage(new Message(MessagesFiscal::PRODUTOS_JA_PARAMETRIZADOS));
            } 
            $this->setInCache(self::PRODUTOS_IGUAIS, $existentes);
    } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    /*private function naoSobreescreve($produtos, $inscricoes)
    {
        $produtosMapiados = array_map(function($produto) {
            return $produto['CDPRODSERV'];
        }, $produtos);
        $existentes = $this->getProdusoParametrizadosInscricoes($produtosMapiados, $inscricoes);
        
        if (count($existentes) == 0) {
            return true;
        } else {
            $this->setInCache(self::PRODUTOS_IGUAIS, $existentes);
            return false;
        }
    }*/
    
    private function getProdusoParametrizadosInscricoes($produtos, $inscricoes)
    {
        $jaParametrizados = array();
        foreach($inscricoes as $inscricao) {
            $filterCriteria = new FilterCriteria($this->dataSource,
            array (
                array (
                        "columnName" => "IDDECLARACAO",
                        "value"     => "RE",
                        "operator"  => "="
                       ),
                array (
                        "columnName" => "CDEMPRESA",
                        "value"     => $inscricao['CDEMPRESA'],
                        "operator"  => "="
                       ),
                array(
                        "columnName" => "CDINSCMUNI",
                        "value"     => $inscricao['CDINSCMUNI'],
                        "operator"  => "="                
                      ),
                array(
                        "columnName" => "CDPRODSERV",
                        "value"     => $produtos,
                        "operator"  => "IN"                
                      )
            ));
            $result = $this->dataSourceManager->findBy($filterCriteria)->getRows();
            $jaParametrizados = array_merge($jaParametrizados, $result);
        }
        
        return $jaParametrizados;
    }
    
    private function getAllProdutos($dados)
    {
        $filterCriteria = new FilterCriteria($this->dataSource,
        array (
            array (
                    "columnName" => "IDDECLARACAO",
                    "value"     => "RE",
                    "operator"  => "="
                   ),
            array (
                    "columnName" => "CDEMPRESA",
                    "value"     => $dados['CDEMPRESA'],
                    "operator"  => "="
                   ),
            array(
                    "columnName" => "CDINSCMUNI",
                    "value"     => $dados['CDINSCMUNI'],
                    "operator"  => "="                
                  )
        ),
        null,
        999999999);
        return $this->dataSourceManager->findBy($filterCriteria)->getRows();
    }
    
    private function getAllInscricoes($dados)
    {
        $filterCriteria = new FilterCriteria('inscrmunicip',
        array (
            array(
                    "columnName" => "CDINSCMUNI",
                    "value"     => $dados['CDINSCMUNI'],
                    "operator"  => "<>"                
                  )
        ),
        null,
        999999999);
        return $this->dataSourceManager->findBy($filterCriteria)->getRows();
    }
    
    private function fazRelacionamentoDiferentes($inscricoes)
    {
        foreach($inscricoes as $inscricao) {
            $produtosIguais = $this->getInCache(self::PRODUTOS_IGUAIS);
            $produtos = $this->getInCache(self::PRODUTOS_BASE);
            try {
                foreach ($produtosIguais as $produtoIgual) {
                    
                    foreach ($produtos as $produto) {
                        if ($produtoIgual['CDPRODSERV'] == $produto['CDPRODSERV']) {
                            $produtoIgual['IDSERVOBRCCIVIL'] = $produto['IDSERVOBRCCIVIL'];
                            $produtoIgual['CDCNO'] = $produto['CDCNO'];
                            $produtoIgual['CDCLASERVMOBEMP'] = $produto['CDCLASERVMOBEMP'];
                            $produtoIgual['IDENTRSAIDOP'] = $produto['IDENTRSAIDOP'];
                        }
                    }
                    
                    $produtoIgual['DTULTATU'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                    $produtoIgual['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                    $produtoIgual['__is_new'] = false;
                    
                    $this->save($produtoIgual);
                }
            } catch (\Exception $e) {
                throw new \Exception(MessagesFiscal::NAO_UPDATAR_ALGUM_PRODUTO.'  Erro: '.$e->getMessage());    
            }
        }
    }
    
    private function fazRelacionamentoIguais($inscricoes)
    {
        $produtos = $this->getInCache(self::PRODUTOS_BASE);

        foreach($inscricoes as $inscricao) {
            try {
                $produtosDaInscricao = $this->getProdutosDaInscricao($inscricao['CDEMPRESA'], $inscricao['CDINSCMUNI']);
                foreach ($produtos as $produto) {
                    $produtoNaoExiste = true;
                    
                    foreach ($produtosDaInscricao as $produtoDaInscricao) {
                        if ($produtoDaInscricao['CDPRODSERV'] == $produto['CDPRODSERV']) {
                            $produtoNaoExiste = false;
                        }
                    }
                    
                    if ($produtoNaoExiste) {
                        $produto['NRSEQPARAMUNISERV'] = $this->newCode->getCodeByTableName('PARAMMUNISERV', $this->environment->getNrOrgTrab(), 1, null, true);
                        
                        $produto['CDEMPRESA'] = $inscricao['CDEMPRESA'];
                        $produto['CDINSCMUNI'] = $inscricao['CDINSCMUNI'];
                        
                        $produto['DTULTATU'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                        $produto['NRORGULTATU'] = $this->environment->getNrOrgTrab();
                        $produto['CDOPERULTATU'] = $this->environment->getCdOperador();
                        $produto['DTINCLUSAO'] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                        $produto['NRORGINCLUSAO'] = $this->environment->getNrOrgTrab();
                        $produto['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
                        $produto['IDATIVO'] = 'S';
                        $produto['__is_new'] = true;
                    
                        $this->save($produto);
                    }
                }
            } catch (\Exception $e) {
                throw new \Exception(MessagesFiscal::NAO_INSERIU_ALGUM_PRODUTO.'  Erro: '.$e->getMessage());    
            }
        }
    }
    
    private function save($data)
    {
        $dataSet = new DataSet($this->dataSource, array($data));
        $this->dataSourceManager->persist($dataSet);
    }
    
    private function setInCache($cacheName, $cacheValue)
    {
        $this->sessionManager->set($cacheName, $cacheValue);
    }
    
    private function getInCache($cacheName)
    {
        return $this->sessionManager->has($cacheName) ? $this->sessionManager->get($cacheName) : array();
    }
    
    private function limpaCache()
    {
        $this->setInCache(self::PRODUTOS_IGUAIS, array());
        $this->setInCache(self::PRODUTOS_DIFRENTES, array());
    }
    
    private function getProdutosDaInscricao($cdempresa, $inscricao)
    {
        $filterCriteria = new FilterCriteria($this->dataSource,
            array (
                array (
                        "columnName" => "IDDECLARACAO",
                        "value"     => "RE",
                        "operator"  => "="
                       ),
                array (
                        "columnName" => "CDEMPRESA",
                        "value"     => $cdempresa,
                        "operator"  => "="
                       ),
                array(
                        "columnName" => "CDINSCMUNI",
                        "value"     => $inscricao,
                        "operator"  => "="                
                      )
            ));
            return $this->dataSourceManager->findBy($filterCriteria)->getRows();
    }
}
